predict.remnlccspec <- function(object,
                                newxdata = NULL,
                                newzdata = NULL,
                                ranefs = NA,
                                level = 0.95,
                                savememory = 10,
                                sims = NA,
                                selected = NULL,
                                equation = NULL
                                ) {
    # Determine number of simulations
    bdraws <- dim(object$sims.list$beta)[1]
    if (is.na(sims)) {
            sims <- nrow(object$sims.list$alpha) 
    }

    # Determine number and size of batches from posterior to process
    if ((savememory>0)&&(savememory<bdraws)) {
        e <- seq(savememory,bdraws,by=savememory)
        s <- seq(1,bdraws,by=savememory)
        if (length(s)>length(e))
            e <- c(e,bdraws)
    } else {
        s <- 1
        e <- bdraws
    }

    # Loop over batches of draws from posterior
    simpchoice <- simpi <- simpmu <- simemu <- NULL

    for (i in 1:length(s)) {
        current <- object
        current$sims.list$alpha <- current$sims.list$alpha[s[i]:e[i],,,drop=FALSE]
        current$sims.list$beta <- current$sims.list$beta[s[i]:e[i],,,drop=FALSE]
        current$sims.list$gamma <- current$sims.list$gamma[s[i]:e[i],,,drop=FALSE]
        
        simulates <- remnlccspecsims(current,
                                     newxdata = newxdata,
                                     newzdata = newzdata,
                                     ranefs = ranefs,
                                     sims = sims,
                                     selected = selected,
                                     equation = equation
                                     )

        #if (is.null(simpchoice))
        #    simpchoice <- simulates$simpchoice
        #else 
        #    simpchoice <- abind(simpchoice,simulates$simpchoice,along=1)
            
        if (is.null(simpi))
            simpi <- simulates$simpi
        else
            simpi <- abind(simpi,simulates$simpi,along=1)
    }
    
    # Calculate summaries of posterior
    if (is.null(equation)) {
      mePr <- apply(simpi,2:3,mean)
      lePr <- apply(simpi,2:3,quantile,probs=(1-level)/2)
      uePr <- apply(simpi,2:3,quantile,probs=(1+level)/2)
      seePr <- apply(simpi,2:3,sd)
    } else {
      mePr <- apply(simpi[,,equation],2,mean)
      lePr <- apply(simpi[,,equation],2,quantile,probs=(1-level)/2)
      uePr <- apply(simpi[,,equation],2,quantile,probs=(1+level)/2)
      seePr <- apply(simpi[,,equation],2,sd)
      simpi <- simpi[,,equation]
    }
    #mpCh <- apply(simpchoice,2:3,mean)

    #lpCh <- apply(simpchoice,2:3,quantile,probs=(1-level)/2)
    #upCh <- apply(simpchoice,2:3,quantile,probs=(1+level)/2)
    #sepCh <- apply(simpchioce,2:3,sd)
    
    # Return results
    prediction <- list(probs=mePr,probslower=lePr,probsupper=uePr,probsse=seePr,
                       #predchoice=mpR,predlower=lpR,predupper=upR,predse=sepR,
                       simpi=simpi)#,#simemu=simemu,
                       #simpchoice=simpchoice)#simpmu=simpmu)
    prediction
}



remnlccspecsims <- function(object,
                            newxdata = NULL,
                            newzdata = NULL,
                            ranefs = NA,
                            sims = 10,
                            context = "observed",  # alternatively, "drawn"
                            selected = NULL,
                            equation = NULL
                            ) {

    bdraws <- dim(object$sims.list$beta)[1]
    C <- dim(object$sims.list$beta)[3]+1

    if (is.null(selected)) {
        ngroups <- dim(object$sims.list$alpha)[2]
        selected <- 1:ngroups
    } else {
        ngroups <- length(selected)
    }
        
    # Optionally estimate counterfactual ranks using newdata
    if (is.null(newxdata)) {
        counterfactual <- FALSE
        newxdata <- object$X
        newzdata <- object$Z

        # Default for fitted values is observed random effect
        if (is.na(ranefs))
          ranefs <- "observed"
      } else {
        counterfactual <- TRUE
        
        # Mean center newdata if necessary
        if (object$meancenter) {
            newxdata <- newxdata - matrix(object$Xmeans,nrow=nrow(newxdata),ncol=ncol(newxdata),byrow=TRUE)
            for (c in 1:(C-1))
                newzdata[,,c] <- (newzdata[,,c]
                                  - matrix(object$Zmeans[c,],nrow=nrow(newzdata),ncol=ncol(newzdata),byrow=TRUE))
        }
        # Default for counterfactuals is to sample random effects from all their posteriors
        if (is.na(ranefs))
            ranefs <- "drawn"
    }

    simemu <- simpi <- array(NA,dim=c(bdraws,nrow(newxdata),C))
    simpmu <- simpchoice <- array(NA,dim=c(bdraws*sims,nrow(newxdata),C))              
    if (ranefs=="drawn") {                        
      if (is.null(equation))
        cloop <- 1:(C-1)
      else
        cloop <- equation
      for (i in 1:nrow(newxdata)) {

            # Make expected pi's
            simemu[,i,C] <- rep(1,bdraws)

            
            for (c in cloop) {

                if (dim(object$sims.list$beta)[2]==1) {
                    xb <- object$sims.list$beta[,1,c]*newxdata[i]
                } else {
                    xb <- object$sims.list$beta[,,c]%*%newxdata[i,]
                }

                if (dim(object$sims.list$gamma)[2]==1) {
                    zg <- object$sims.list$gamma[,1,c]*newzdata[i,,c]
                } else {
                    zg <- object$sims.list$gamma[,,c]%*%newzdata[i,,c]
                }
                simemu[,i,c] <- exp(xb + zg + apply(object$sims.list$alpha[,selected,c],1,mean))
            }
            
            if (is.null(equation)) {
              edenom <- apply(simemu[,i,],1,sum)
              for (c in 1:C) {
                simpi[,i,c] <- simemu[,i,c]/edenom
              }            
            } else {
              simpi[,i,equation] <- simemu[,i,equation]/(1+simemu[,i,equation])
            }

            # Make predicted choices
            simpmu[,i,C] <- rep(1,bdraws*sims)
            bsamp <- rep(1:bdraws,sims)
            asamp <- sample(1:ngroups,bdraws*sims,replace=TRUE)
            for (c in cloop) {

                if (dim(object$sims.list$beta)[2]==1) {
                    xb <- object$sims.list$beta[bsamp,1,c]*newxdata[i]
                } else {
                    xb <- object$sims.list$beta[bsamp,,c]%*%newxdata[i,]
                }

                if (dim(object$sims.list$gamma)[2]==1) {
                    zg <- object$sims.list$gamma[bsamp,1,c]*newzdata[i,,c]
                } else {
                    zg <- object$sims.list$gamma[bsamp,,c]%*%newzdata[i,,c]
                }
                
                simpmu[,i,c] <- exp(xb + zg +
                                    + object$sims.list$alpha[bsamp,selected,c][asamp])
            }
            pdenom <- apply(simpmu[,i,],1,sum)
            for (c in 1:C) {
               # simpchoice[,i,c] <- (simpmu[,c]/pdenom)>XXX
            }

        }
    } else {  # Observed ranefs
      if (is.null(equation))
        cloop <- 1:(C-1)
      else
        cloop <- equation
      for (i in 1:nrow(newxdata)) {
            # Make expected pi's
            simemu[,i,C] <- rep(1,bdraws)
            for (c in cloop) {

              if (dim(object$sims.list$beta)[2]==1) {
                    xb <- object$sims.list$beta[,1,c]*newxdata[i]
                } else {
                    xb <- object$sims.list$beta[,,c]%*%newxdata[i,]
                }

                if (dim(object$sims.list$gamma)[2]==1) {
                    zg <- object$sims.list$gamma[,1,c]*newzdata[i,,c]
                } else {
                    zg <- object$sims.list$gamma[,,c]%*%newzdata[i,,c]
                }                
                simemu[,i,c] <- exp(xb + zg +
                                    + object$sims.list$alpha[,i,c])
            }
            if (is.null(equation)) {
              edenom <- apply(simemu[,i,],1,sum)
              for (c in 1:C) {
                simpi[,i,c] <- simemu[,i,c]/edenom
              }            
            } else {
              simpi[,i,equation] <- simemu[,i,equation]/(1+simemu[,i,equation])
            }
            
            
            # Make predicted choices
            simpmu[,i,C] <- rep(1,bdraws*sims)
            bsamp <- rep(1:bdraws,sims)
            asamp <- sample(1:ngroups,bdraws*sims,replace=TRUE)
            for (c in cloop) {

                if (dim(object$sims.list$beta)[2]==1) {
                    xb <- object$sims.list$beta[bsamp,1,c]*newxdata[i]
                } else {
                    xb <- object$sims.list$beta[bsamp,,c]%*%newxdata[i,]
                }

                if (dim(object$sims.list$gamma)[2]==1) {
                    zg <- object$sims.list$gamma[bsamp,1,c]*newzdata[i,,c]
                } else {
                    zg <- object$sims.list$gamma[bsamp,,c]%*%newzdata[i,,c]
                }
                
                simpmu[,i,c] <- exp(xb + zg +
                                    + object$sims.list$alpha[bsamp,i,c])
            }
            pdenom <- apply(simpmu[,i,],1,sum)
            for (c in 1:C) {
               # simpchoice[,i,c] <- (simpmu[,c]/pdenom)>XXX
            }   
        }
    }   

    list(simpi=simpi,simemu=simemu,
         simpchoice=simpchoice,simpmu=simpmu)    
}




plotchoicerl <- function(object,
                         names,
                         n,
                         interval = "confidence", # alternative is prediciton
                         limits=NA,
                         at=NA,
                         thresholds=NULL,
                         comparerank=NULL,
                         comparelower=NULL,
                         compareupper=NULL,
                         outfile="echoice",
                         maintitle="",
                         rows=50,
                         width=7,
                         col="black",
                         plotlist=NULL,
                         pch=19,
                         lwd=0.5,
                         spacetraces=0.8,
                         vmark=50,
                         plotitems=NULL,
                         entryheight=1/6,
                         subentryheight=0.6,
                         sublabels = NULL,
                         sublabeloffset=0.025
                         ) {

    
  
    if (identical(interval,"confidence")) {
        x <- cbind(as.matrix(object$probs)[,1],as.matrix(object$probslower)[,1],as.matrix(object$probsupper)[,1])
        
    } else {
        x <- cbind(as.matrix(object$predrank)[,1],as.matrix(object$predlower)[,1],as.matrix(object$predupper)[,1])
     
    }

    if (!is.null(plotitems)) {
        x <- x[plotitems,]
    }
    
    namesx <- as.character(names[order(x[,1])])
   
    if (!is.null(comparerank)) {
        if (!is.null(plotitems)) {
            comparerank <- as.matrix(comparerank)[plotitems,]
        }
        
        comparerank <- comparerankx <- as.matrix(comparerank)
        for (i in 1:ncol(comparerank)) {
            comparerankx[,i] <- comparerank[order(x[,1]),i]
           
        }
    }
    if (!is.null(comparelower)) {
        if (!is.null(plotitems)) {
            comparelower <- as.matrix(comparelower)[plotitems,]
        }
        comparelower <- comparelowerx <- as.matrix(comparelower)
        for (i in 1:ncol(comparelower)) {
            comparelowerx[,i] <- comparelower[order(x[,1]),i]  
        }
    }
    if (!is.null(compareupper)) {
        if (!is.null(plotitems)) {
            compareupper <- as.matrix(compareupper)[plotitems,]
        }
        compareupper <- compareupperx <- as.matrix(compareupper)
        for (i in 1:ncol(compareupper)) {
            compareupperx[,i] <- compareupper[order(x[,1]),i]  
        }
    }
    sortx <- x[order(x[,1]),]
    if (is.na(limits)) 
        limits <- c(0,100)
    if (is.na(at)) {
        at <- c(0,20,40,60,80,100)
    }
    nplots <- ((nrow(x)%%rows)>0) + nrow(x)%/%rows


    if (!is.null(thresholds)) {
        thresholds <- 1-(rev(sort(thresholds))/n)
    }

    dataout <- cbind(namesx,sortx)
    varsout <- c("case","x","lowerx","upperx")
    if (!is.null(comparerank)) {
        dataout <- cbind(dataout,comparerankx)
        for (ic in 1:ncol(comparerankx)) {
            varsout <- c(varsout,paste("x",ic,sep=""))
        }
    }
    if (!is.null(comparelower)) {
        dataout <- cbind(dataout,comparelowerx)
        for (ic in 1:ncol(comparelowerx)) {
            varsout <- c(varsout,paste("lowerx",ic,sep=""))
        }
    }
    if (!is.null(compareupper)) {
        dataout <- cbind(dataout,compareupperx)
        for (ic in 1:ncol(compareupperx)) {
            varsout <- c(varsout,paste("upperx",ic,sep=""))
        }
    }
    dataout <- as.data.frame(dataout)
    colnames(dataout) <- varsout
    
    write.csv(dataout,
              file=paste(outfile,".csv",sep="")
              )
    
    
    for (i in 1:nplots) {
        s <- rows*(i-1) + 1
        e <- min(nrow(x), rows*i)
        outfile0 <- paste(outfile,"_",i,sep="")
        if (nplots>1) {
            maint <- paste(maintitle,", ",i,"/",nplots,sep="")
        } else {
            maint <- maintitle
        }

        lab.x <- at
        xplot <- sortx[s:e,1]
        lowerplot <- sortx[s:e,2]
        upperplot <- sortx[s:e,3]

        trace0 <- ropeladder(x=xplot,
                             lower=lowerplot,
                             upper=upperplot,
                             labels=namesx,
                             col=col[1],
                             pch=pch[,1],
                             lwd=lwd,
                             plot=1,
                             entryheight=entryheight,
                             subentryheight=subentryheight,
                             sublabels = sublabels[1],
                             sublabelsyoffset = sublabeloffset
                             
                             )
        
        if (!is.null(comparerank)) {
            for (k in 1:ncol(as.matrix(comparerankx))) {


                if (!is.null(comparelower)) {

                  if (!is.null(sublabels)) {
                    assign(paste("trace",k,sep=""),
                           ropeladder(x=as.matrix(comparerankx)[s:e,k],
                                      lower=as.matrix(comparelowerx)[s:e,k],
                                      upper=as.matrix(compareupperx)[s:e,k],
                                      labels=namesx,
                                      col=col[k+1],
                                      pch=pch[,k+1],
                                      lwd=lwd,
                                      plot=1,
                                      entryheight=entryheight,
                                      subentryheight=subentryheight,
                                      sublabels = sublabels[k+1],
                                      sublabelsyoffset = sublabeloffset
                                      )
                           )
                  } else {
                    assign(paste("trace",k,sep=""),
                           ropeladder(x=as.matrix(comparerankx)[s:e,k],
                                      lower=as.matrix(comparelowerx)[s:e,k],
                                      upper=as.matrix(compareupperx)[s:e,k],
                                      labels=namesx,
                                      col=col[k+1],
                                      pch=pch[,k+1],
                                      lwd=lwd,
                                      plot=1,
                                      entryheight=entryheight,
                                      subentryheight=subentryheight
                                      )
                           )
                  }
                } else {
                  if (!is.null(sublabels)) {
                    assign(paste("trace",k,sep=""),
                           ropeladder(x=as.matrix(comparerankx)[s:e,k],
                                       labels=namesx,
                                      col=col[k],
                                      pch=pch,
                                      lwd=lwd,
                                      plot=1,
                                      entryheight=entryheight,
                                      subentryheight=subentryheight,
                                      sublabels = sublabels[k+1],
                                      sublabelsyoffset = sublabeloffset
                                      )
                           )
                  } else {
                     assign(paste("trace",k,sep=""),
                           ropeladder(x=as.matrix(comparerankx)[s:e,k],
                                       labels=namesx,
                                    col=col[k],
                                      pch=pch,
                                      lwd=lwd,
                                      plot=1,
                                      entryheight=entryheight,
                                      subentryheight=subentryheight
                                      )
                           )
                  }

                }
            }
        }


        #xsublabel <- c(xplot[1])
        #ysublabel <- c(.9,.85,.80)
        #for (i in 1:ncol(comparerankx)) {
        # xsublabel <- c(xsublabel,comparerankx[s,i])
        #}
        #sublabels <- textTile(x = xsublabel,
        #                      y = ysublabel,
        #                      labels= c("state", "region", "local"),
        #                      col = col,
        #                      cex = 0.5,
        #                      plot = 1)

# Call ropeladder
        tile::tile(# Required input
             trace0,
             trace1,
             trace2,
                   sublabels,
           #plottitle=list(fontface="bold",labels=c("Turnout decision")),
                                     #  Plot titles (appear above plots)
             #maintitle=list(fontface="bold",labels=c(maint)),
                                        #  Overtitles (appear above plot matrix)
             topaxis=list(at=at,add=TRUE),
             topaxistitle=list(labels="Probability of allocation"),
                                        # X-axis controls
             xaxis=list(at=at),
             xaxistitle=list(labels="Probability of allocation"),
                                        # Top-axis controls
             limits=limits,           # Plot region boundaries
             width=list(null=2.5), #titleboxes=TRUE, axisboxes=TRUE,
                   height=list(xaxistitle=2,topaxistitle=2),
                   gridlines=list(type="xt"),
             output = list(file=outfile0,type="pdf")#,
     
                                        #gridlines=list(type="x"),          # Gridlines controls
                                        #vertmark=list(at=100*(1-185/n)),       # Add vertical lines
             #entryheight = 0.33,
             #shadebox=NA,              #  Add gray shading of point estimate range?
             #RxC=c(1,1)#,                #  Number of Rows and Columns of plots to make
             #spacetraces=spacetraces
             )

        #outx <- c(outx,xplot)
        #outlower <- c(outlower,lowerplot)
        #outupper <- c(outupper,upperplot)
        #outlabels <- c(outlabels,namespx[s:e])
        
    }

    # Save plotted points
    
    #write.csv()
    
            
    NULL
    
}
